@@ -33,6 +33,16 @@
 
 #include "gdm-common.h"
 
+#ifndef HAVE_MKDTEMP
+#include "mkdtemp.h"
+#endif
+
+const char *
+gdm_make_temp_dir (char *template)
+{
+        return mkdtemp (template);
+}
+
 gboolean
 gdm_is_version_unstable (void)
 {
@@ -372,15 +382,15 @@ _read_bytes (int      fd,
         if (premature_eof) {
                 g_set_error (error,
                              G_FILE_ERROR,
-                             g_file_error_from_errno (ENODATA),
-                             g_strerror (ENODATA));
+                             G_FILE_ERROR_FAILED,
+                             "No data available");
 
                 return FALSE;
         } else if (bytes_left_to_read > 0) {
                 g_set_error (error,
                              G_FILE_ERROR,
                              g_file_error_from_errno (errno),
-                             g_strerror (errno));
+                             "%s", g_strerror (errno));
                 return FALSE;
         }
 
