@@ -50,6 +50,7 @@ struct GdmLayoutOptionWidgetPrivate
 
 enum {
         LAYOUT_ACTIVATED,
+        DIALOG_HIDDEN,
         NUMBER_OF_SIGNALS
 };
 
@@ -97,6 +98,7 @@ gdm_layout_option_widget_hide_dialog (GdmLayoutOptionWidget *widget)
 {
         gtk_widget_destroy (widget->priv->dialog);
         widget->priv->dialog = NULL;
+        g_signal_emit (G_OBJECT (widget), signals [DIALOG_HIDDEN], 0);
 }
 
 static void
@@ -173,6 +175,16 @@ gdm_layout_option_widget_class_init (GdmLayoutOptionWidgetClass *klass)
                                                   G_TYPE_NONE,
                                                   0);
 
+        signals[DIALOG_HIDDEN] = g_signal_new ("dialog-hidden",
+                                               G_TYPE_FROM_CLASS (object_class),
+                                               G_SIGNAL_RUN_FIRST,
+                                               G_STRUCT_OFFSET (GdmLayoutOptionWidgetClass, dialog_hidden),
+                                               NULL,
+                                               NULL,
+                                               g_cclosure_marshal_VOID__VOID,
+                                               G_TYPE_NONE,
+                                               0);
+
         g_type_class_add_private (klass, sizeof (GdmLayoutOptionWidgetPrivate));
 }
 
@@ -220,7 +232,7 @@ gdm_layout_option_widget_init (GdmLayoutOptionWidget *widget)
                                     /* translators: This brings up a dialog of
                                      * available keyboard layouts
                                      */
-                                    C_("keyboard", "Other..."),
+                                    C_("keyboard", "Other…"),
                                     _("Choose a keyboard layout from the "
                                       "full list of available layouts."),
                                     GDM_OPTION_WIDGET_POSITION_BOTTOM);
@@ -251,7 +263,7 @@ gdm_layout_option_widget_new (void)
         GObject *object;
 
         object = g_object_new (GDM_TYPE_LAYOUT_OPTION_WIDGET,
-                               "label-text", _("Keyboard"),
+                               "label-text", _("_Keyboard:"),
                                "icon-name", "preferences-desktop-keyboard",
                                "max-item-count", 8,
                                NULL);
